# babel-plugin-transform-function-composition

[![Travis build status](http://img.shields.io/travis/gajus/babel-plugin-transform-function-composition/master.svg?style=flat-square)](https://travis-ci.org/gajus/babel-plugin-transform-function-composition)
[![NPM version](http://img.shields.io/npm/v/babel-plugin-transform-function-composition.svg?style=flat-square)](https://www.npmjs.org/package/babel-plugin-transform-function-composition)
[![Canonical Code Style](https://img.shields.io/badge/code%20style-canonical-blue.svg?style=flat-square)](https://github.com/gajus/canonical)

Transpiles [function-bind call expressions](https://github.com/babel/babel/tree/master/packages/babel-plugin-syntax-function-bind) to partially applied call expressions. Uses the invocation context to set the first parameter of the callee function.

* Coming from Clojure? This transpiler enables a syntactic suger for [thread-first](http://clojure.org/guides/threading_macros#thread-first)

> ***BIG WARNING***: This is a proof-of-concept. See [Motivation](#motivation).

* [Example transpilation](#example-transpilation)
* [Motivation](#motivation)
* [Difference from This-Binding Syntax proposal](#difference-from-this-binding-syntax-proposal)
  * [Benefits](#benefits)
  * [Conflicting syntax](#conflicting-syntax)
* [Usage examples](#usage-examples)
  * [Using existing functional programming utilities](using-existing-functional-programming-utilities)
  * [Using utility functions to construct Promises](using-utility-functions-to-construct-promises)

## Example transpilation

Input:

```js
apple
  ::foo('foo parameter 0', 'foo parameter 1')
  ::bar('bar parameter 0');
```

Output:

```js
bar('bar parameter 0', foo('foo parameter 0', 'foo parameter 1', apple));
```

## Motivation

Add a syntactic sugar that enables chaining of curried functions. In a functional pipeline, participating functions expect the input of the function to be the last parameter.

In effect, `babel-plugin-transform-function-composition` is equivalent to using [`R.pipe`](http://ramdajs.com/docs/#pipe) in [Ramda](http://ramdajs.com/).

## Difference from This-Binding Syntax proposal

[ECMAScript This-Binding Syntax](https://github.com/tc39/proposal-bind-operator) proposal introduces a new operator `::` which performs `this` binding and method extraction, i.e.

The following input:

```js
foo::bar()::baz()
```

Becomes the following output:

```js
var _context;

(_context = (_context = foo, bar).call(_context), baz).call(_context);
```

`babel-plugin-transform-function-composition` uses the `::`` operator to create a partially applied function such that the left hand side of the operator is set as the the first parameter to the target function on the right hand side, i.e.

The following input:

```js
foo::bar()::baz()
```

Becomes the following output:

```js
baz(bar(foo));
```

#### Benefits

TODO

#### Conflicting syntax

At the time of this writing, [ECMAScript This-Binding Syntax](https://github.com/tc39/proposal-bind-operator) remains in stage 0.

See [What's keeping this from Stage 1?](https://github.com/tc39/proposal-bind-operator/issues/24).

### Proposal

TODO

## Usage examples

#### Using existing functional programming utilities

```js
import {
  assocPath
} from 'ramda';

({
  name: 'babel-plugin-transform-function-composition'
})
  ::assocPath(['repository', 'type'], 'git')
  ::assocPath(['repository', 'url'], 'https://github.com/gajus/babel-plugin-transform-function-composition');

// {
//   name: 'babel-plugin-transform-function-composition',
//   repository: {
//     type: 'git',
//     url: 'https://github.com/gajus/babel-plugin-transform-function-composition'
//   }
// }
```

#### Using utility functions to construct Promises

```js
const map = (callback, promise) => {
  return promise
    .then((values) => {
      return values.map(callback);
    });
};

Promise
  .resolve([
    'foo',
    'bar',
    'baz'
  ])
  ::map((currentValue, index) => {
    return index + ':' + currentValue;
  })
  .then((values) => {
    values;

    // [
    //   '0:foo',
    //   '1:bar',
    //   '2:baz'
    // ]
  });
```
